import dateutil.parser
from datetime import date
from .. import constants


class ProfileDescriptor(object):
    # this is a python descriptor that allows for
    # dynamic updating of profile data
    def __init__(self, id):
        self.id = id

    def __get__(self, instance, owner):
        if hasattr(self, 'value'):
            return self.value
        else:
            return instance._data[self.id]

    def __set__(self, instance, value):
        profile = {}
        for key in constants.UPDATABLE_FIELDS:
            profile[key] = getattr(instance, key)
        profile['gender'] = constants.GENDER_MAP_REVERSE[profile['gender']]
        profile['interested_in'] = map(
            lambda x: constants.GENDER_MAP_REVERSE[x],
            profile['interested_in']
            )
        profile[self.id] = value
        instance.__init__(instance._api.update_profile(profile), instance._api)
        self.value = value


class GenderDescriptor(ProfileDescriptor):
    # makes gender human readable

    def __get__(self, instance, owner):
        gender = super(GenderDescriptor, self).__get__(instance, owner)
        return constants.GENDER_MAP[gender]

    def __set__(self, instance, value):
        gender = constants.GENDER_MAP_REVERSE[value]
        return super(GenderDescriptor, self).__get__(instance, gender)


class InterestedInDescriptor(ProfileDescriptor):
    # makes gender human readable
    def __get__(self, instance, owner):
        interested_in = super(InterestedInDescriptor, self).\
            __get__(instance, owner)
        return map(lambda x: constants.GENDER_MAP[x], interested_in)

    def __set__(self, instance, value):
        interested_in = map(lambda x: constants.GENDER_MAP_REVERSE[x], value)
        return super(InterestedInDescriptor, self).\
            __get__(instance, interested_in)


class Profile(object):
    bio = ProfileDescriptor('bio')
    discoverable = ProfileDescriptor('discoverable')
    distance_filter = ProfileDescriptor('distance_filter')
    age_filter_min = ProfileDescriptor('age_filter_min')
    age_filter_max = ProfileDescriptor('age_filter_max')
    interested_in = InterestedInDescriptor('interested_in')
    gender = GenderDescriptor('gender')

    def __init__(self, data, api):
        self.id = data['_id']
        self._api = api
        self.create_date = data['create_date']
        self.photos = map(lambda photo: str(photo['url']), data['photos'])
        self.ping_time = data['ping_time']
        self.name = data['name']
        today = date.today()
        self.create_date = dateutil.parser.parse(self.create_date)
        self._data = data

    def __repr__(self):
        return self.name



import dateutil.parser
from datetime import date
from .. import constants
from .message import Message


class User(object):
    def __init__(self, data, session):
        self.id = data['_id']
        try:
            self.distance = data['distance_mi']
        except KeyError:
            self.distance = data['distance_km']
        self.common_friends = data['common_friends']
        self.common_likes = data['common_likes']
        self.bio = data['bio']
        self._session = session
        self.gender = constants.GENDER_MAP[int(data['gender'])]
        self.birth_date = data['birth_date']
        self.photos = map(lambda photo: str(photo['url']), data['photos'])
        self.ping_time = data['ping_time']
        self.name = data['name']
        today = date.today()
        self.birth_date = dateutil.parser.parse(self.birth_date)
        self.age = (today.year - self.birth_date.year -
                    ((today.month, today.day) <
                     (self.birth_date.month, self.birth_date.day)))

    def __repr__(self):
        return self.name

    def report(self, cause):
        return self._session._api.report(self.id, cause)


class Hopeful(User):
    def like(self):
        return self._session._api.like(self.id)['match']

    def dislike(self):
        return self._session._api.dislike(self.id)['match']


class Match(User):
    def __init__(self, match, _session):
        user_data = _session._api.user_info(match['person']['_id'])['results']
        user_data['_id'] = match['person']['_id']
        super(Match, self).__init__(user_data, _session)
        self.messages = map(lambda message: Message(message, user=self),
                            match['messages'])

    def message(self, body):
        return self._session._api.message(self.id, body)['_id']



import dateutil.parser


class Message(object):

    def __init__(self, data, user=None):
        if isinstance(data, str):
            self.body = data
        else:
            self._data = data
            self.body = data['message']
        if user:
            if data['from'] == user.id:
                self.sender = user
            if data['to'] == user.id:
                self.to = user
            if data['from'] == user._session.profile.id:
                self.sender = user._session.profile
            if data['to'] == user._session.profile.id:
                self.to = user._session.profile
                self.sent = dateutil.parser.parse(data['sent_date'])

    def __repr__(self):
        return self.body

    def __str__(self):
        return self.body



from . import api
from . import models


class Session(object):

    def __init__(self, facebook_id, facebook_token):
        self._api = api.TinderAPI()
        # perform authentication
        self._api.auth(facebook_id, facebook_token)
        self.profile = models.Profile(self._api.profile(), self)

    def nearby_users(self):
        return map(lambda user: models.Hopeful(user, self),
                   self._api.recs()['results'])

    def update_location(self, latitude, longitude):
        return self._api.ping(latitude, longitude)

    def matches(self):
        return map(lambda match: models.Match(match, self),
                   self._api.matches())



API_BASE = 'https://api.gotinder.com'

USER_AGENT = 'Tinder Android Version 4.1.3'

HEADERS = {
    "User-Agent": USER_AGENT,
    "os_version": "19",
    "app-version": "759",
    "platform": "android",
    "Content-type": "application/json; charset=utf-8"
}

GENDER_MAP = ["male", "female"]

GENDER_MAP_REVERSE = {"male": 0, "female": 1}

UPDATABLE_FIELDS = [
    'gender', 'age_filter_min', 'age_filter_max',
    'distance_filter', 'age_filter_min', 'bio', 'interested_in'
]



import os
from setuptools import setup, find_packages


version = '0.0.4'

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as file:
    long_description = file.read()

setup(
    name="pynder",
    version=version,
    packages=find_packages(exclude=('examples')),
    install_requires=['requests', 'python-dateutil'],
    package_data={'': ['*.rst']},
    author="Charlie Wolf",
    author_email="charlie@wolf.is",
    description="A client for the tinder api",
    license="MIT",
    keywords=["tinder", "online dating"],
    url="https://github.com/charliewolf/pynder",
    long_description=long_description,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2",
        "Topic :: Utilities",
    ],
)



import requests
import json
from . import constants
from . import errors


class TinderAPI(object):

    def __init__(self):
        self._session = requests.Session()
        self._session.headers.update(constants.HEADERS)
        self._token = None

    def _url(self, path):
        return constants.API_BASE + path

    def auth(self, facebook_id, facebook_token):
        data = json.dumps({"facebook_id": str(facebook_id),
                          "facebook_token": facebook_token})
        result = self._session.post(self._url('/auth'), data=data).json()
        if 'token' not in result:
            raise errors.RequestError("Couldn't authenticate")
        self._token = result['token']
        self._session.headers.update({"X-Auth-Token": str(result['token'])})
        return result

    def _get(self, url):
        if not hasattr(self, '_token'):
            raise errors.InitializationError

        result = self._session.get(self._url(url)).json()
        if ('status' in result) and result['status'] != 200:
            raise errors.RequestError(result['status'])
        return result

    def _post(self, url, data={}):
        if not hasattr(self, '_token'):
            raise errors.InitializationError

        result = self._session.post(self._url(url),
                                    data=json.dumps(data)).json()
        if ('status' in result) and result['status'] != 200:
            raise errors.RequestError(result['status'])
        return result

    def updates(self):
        return self._post("/updates")

    def recs(self):
        return self._get("/user/recs")

    def matches(self):
        return self.updates()['matches']

    def profile(self):
        return self._get("/profile")

    def update_profile(self, profile):
        return self._post("/profile", profile)

    def like(self, user):
        return self._get("/like/{}".format(user))

    def dislike(self, user):
        return self._get("/pass/{}".format(user))

    def message(self, user, body):
        return self._post("/user/matches/{}".format(user),
                          {"message": str(body)})

    def report(self, user, cause=1):
        return self._post("/report/" + user, {"cause": cause})

    def user_info(self, user_id):
        return self._get("/user/"+user_id)

    def ping(self, lat, lon):
        return self._post("/user/ping", {"lat": lat, "lon": lon})
